/* This file was generated by plugin 'Nordic Semiconductor nRF5x v.1.2.2' (BDS version 1.1.3135.0) */

#include "ble_myprofile.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_srv_common.h"
#include "app_util.h"
#include "app_util_bds.h"

#define OPCODE_LENGTH 1 /**< Length of opcode inside MyProfile packet. */
#define HANDLE_LENGTH 2 /**< Length of handle inside MyProfile packet. */

/* TODO Consider changing the max values if encoded data for characteristic/descriptor is fixed length */ 
#define MAX_MYCHARACTERISTIC_LEN (BLE_L2CAP_MTU_DEF - OPCODE_LENGTH - HANDLE_LENGTH) /**< Maximum size of a transmitted MyCharacteristic. */ 

/**@brief Function for encoding MyCharacteristic.
 *
 * @param[in]   p_mycharacteristic              MyCharacteristic characteristic structure to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
static uint8_t mycharacteristic_encode(ble_myprofile_mycharacteristic_t * p_mycharacteristic, uint8_t * encoded_buffer)
{
    uint8_t len = 0; 
    encoded_buffer[len++] = p_mycharacteristic->value;
    return len;
}

/**@brief Function for decoding MyCharacteristic.
 *
 * @param[in]   data_len              Length of the field to be decoded.
 * @param[in]   p_data                Buffer where the encoded data is stored.
 * @param[out]  p_write_val           Decoded data.
 *
 * @return      Length of the decoded field.
 */
static uint8_t mycharacteristic_decode(uint8_t data_len, uint8_t * p_data, ble_myprofile_mycharacteristic_t * p_write_val)
{
    uint8_t pos = 0;
    p_write_val->value = p_data[pos++]; 

    return pos;
} 

/**@brief Function for handling the Connect event.
 *
 * @param[in]   p_myprofile       MyProfile Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_connect(ble_myprofile_t * p_myprofile, ble_evt_t * p_ble_evt)
{
    p_myprofile->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}

/**@brief Function for handling the Disconnect event.
 *
 * @param[in]   p_myprofile       MyProfile Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_disconnect(ble_myprofile_t * p_myprofile, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_myprofile->conn_handle = BLE_CONN_HANDLE_INVALID;
}

/**@brief Function for handling the Write event.
 *
 * @param[in]   p_myprofile       MyProfile Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_write(ble_myprofile_t * p_myprofile, ble_gatts_evt_write_t * p_ble_evt)
{
    
    if(p_ble_evt->handle == p_myprofile->mycharacteristic_handles.cccd_handle)
    {
        if(p_myprofile->evt_handler != NULL)
        {
            ble_myprofile_evt_t evt;
            evt.evt_type = BLE_MYPROFILE_MYCHARACTERISTIC_EVT_CCCD_WRITE;
            bds_uint16_decode(p_ble_evt->len, p_ble_evt->data, &evt.params.cccd_value);
            p_myprofile->evt_handler(p_myprofile, &evt);
        }
    } 
    if(p_ble_evt->handle == p_myprofile->mycharacteristic_handles.value_handle)
    {
        if(p_myprofile->evt_handler != NULL)
        {
            ble_myprofile_evt_t evt;
            evt.evt_type = BLE_MYPROFILE_MYCHARACTERISTIC_EVT_WRITE;
            mycharacteristic_decode(p_ble_evt->len, p_ble_evt->data, &evt.params.mycharacteristic);
            p_myprofile->evt_handler(p_myprofile, &evt);
        }
    }
}

/**@brief Authorize WRITE request event handler.
 *
 * @details Handles WRITE events from the BLE stack.
 *
 * @param[in]   p_sc_ctrlpt  SC Ctrlpt structure.
 * @param[in]   p_gatts_evt  GATTS Event received from the BLE stack.
 *
 */
static void on_rw_authorize_request(ble_myprofile_t * p_myprofile, ble_gatts_evt_t * p_gatts_evt)
{
    ble_gatts_evt_rw_authorize_request_t * p_auth_req = &p_gatts_evt->params.authorize_request;
    if (p_auth_req->type == BLE_GATTS_AUTHORIZE_TYPE_WRITE)
    {
        if (   (p_gatts_evt->params.authorize_request.request.write.op
                != BLE_GATTS_OP_PREP_WRITE_REQ)
            && (p_gatts_evt->params.authorize_request.request.write.op
                != BLE_GATTS_OP_EXEC_WRITE_REQ_NOW)
            && (p_gatts_evt->params.authorize_request.request.write.op
                != BLE_GATTS_OP_EXEC_WRITE_REQ_CANCEL)
           )
        {
        
            if (p_auth_req->request.write.handle == p_myprofile->mycharacteristic_handles.value_handle)
            {
                on_write(p_myprofile, &p_auth_req->request.write);
            }
        }
    }
}

/**@brief Function for handling BLE events.
 *
 * @param[in]   p_myprofile       MyProfile Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
void ble_myprofile_on_ble_evt(ble_myprofile_t * p_myprofile, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_myprofile, p_ble_evt);
            break;
        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_myprofile, p_ble_evt);
            break;
        case BLE_GATTS_EVT_WRITE:
            on_write(p_myprofile, &p_ble_evt->evt.gatts_evt.params.write);
            break;
         case BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST:
            on_rw_authorize_request(p_myprofile, &p_ble_evt->evt.gatts_evt);
            break;
        default:
            //No implementation needed.
            break;
    }
}

/**@brief Function for initializing the MyProfile. */
uint32_t ble_myprofile_init(ble_myprofile_t * p_myprofile, const ble_myprofile_init_t * p_myprofile_init)
{
    uint32_t err_code;
    ble_uuid_t ble_uuid;

    // Initialize service structure
    p_myprofile->evt_handler = p_myprofile_init->evt_handler;
    p_myprofile->conn_handle = BLE_CONN_HANDLE_INVALID;
    
    // Add a custom base UUID.
    ble_uuid128_t bds_base_uuid = {{0x14, 0x40, 0x4B, 0x86, 0xD7, 0x62, 0x28, 0x83, 0xDE, 0x4E, 0x2C, 0xAF, 0x00, 0x00, 0x05, 0xB3}};
    uint8_t       uuid_type;
    err_code = sd_ble_uuid_vs_add(&bds_base_uuid, &uuid_type);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    ble_uuid.type = uuid_type;
    ble_uuid.uuid = 0xB68D;
        
    // Add service
    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_myprofile->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    } 

    // Add MyCharacteristic characteristic
    ble_myprofile_mycharacteristic_t mycharacteristic_initial_value = p_myprofile_init->ble_myprofile_mycharacteristic_initial_value; 

    uint8_t mycharacteristic_encoded_value[MAX_MYCHARACTERISTIC_LEN];
    ble_add_char_params_t add_mycharacteristic_params;
    memset(&add_mycharacteristic_params, 0, sizeof(add_mycharacteristic_params));
    
    add_mycharacteristic_params.uuid                = 0x84A9;
    add_mycharacteristic_params.uuid_type           = ble_uuid.type; 
    add_mycharacteristic_params.max_len             = MAX_MYCHARACTERISTIC_LEN;
    add_mycharacteristic_params.init_len            = mycharacteristic_encode(&mycharacteristic_initial_value, mycharacteristic_encoded_value);
    add_mycharacteristic_params.p_init_value        = mycharacteristic_encoded_value; 
    add_mycharacteristic_params.char_props.notify   = 1; 
    add_mycharacteristic_params.char_props.indicate = 1; 
    add_mycharacteristic_params.char_props.read     = 1; 
    add_mycharacteristic_params.read_access         = SEC_OPEN; 
    add_mycharacteristic_params.char_props.write    = 1; 
    add_mycharacteristic_params.write_access        = SEC_OPEN; 
    add_mycharacteristic_params.cccd_write_access   = SEC_OPEN;
    // 1 for variable length and 0 for fixed length.
    add_mycharacteristic_params.is_var_len          = 1; 

    err_code = characteristic_add(p_myprofile->service_handle, &add_mycharacteristic_params, &(p_myprofile->mycharacteristic_handles));
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    } 

		
		
    return NRF_SUCCESS;
}

/**@brief Function for setting the MyCharacteristic. */
uint32_t ble_myprofile_mycharacteristic_set(ble_myprofile_t * p_myprofile, ble_myprofile_mycharacteristic_t * p_mycharacteristic)
{
    ble_gatts_value_t gatts_value;
    uint8_t encoded_value[MAX_MYCHARACTERISTIC_LEN];

    // Initialize value struct.
    memset(&gatts_value, 0, sizeof(gatts_value));

    gatts_value.len     = mycharacteristic_encode(p_mycharacteristic, encoded_value);
    gatts_value.offset  = 0;
    gatts_value.p_value = encoded_value;

    return sd_ble_gatts_value_set(p_myprofile->conn_handle, p_myprofile->mycharacteristic_handles.value_handle, &gatts_value);
}

/**@brief Function for sending the MyCharacteristic. */
uint32_t ble_myprofile_mycharacteristic_notify(ble_myprofile_t * p_myprofile, ble_myprofile_mycharacteristic_t * p_mycharacteristic)
{
    uint32_t err_code = NRF_SUCCESS;

    if (p_myprofile->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        ble_gatts_hvx_params_t hvx_params;
        uint8_t encoded_value[MAX_MYCHARACTERISTIC_LEN];
        uint16_t hvx_len;

        // Initialize value struct.
        memset(&hvx_params, 0, sizeof(hvx_params));

        hvx_len           = mycharacteristic_encode(p_mycharacteristic, encoded_value);
        hvx_params.handle = p_myprofile->mycharacteristic_handles.value_handle;
        hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.p_len  = &hvx_len;
        hvx_params.offset = 0;
        hvx_params.p_data = encoded_value;

        err_code = sd_ble_gatts_hvx(p_myprofile->conn_handle, &hvx_params);
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
}

/**@brief Function for sending the MyCharacteristic. */
uint32_t ble_myprofile_mycharacteristic_indicate(ble_myprofile_t * p_myprofile, ble_myprofile_mycharacteristic_t * p_mycharacteristic)
{
    uint32_t err_code = NRF_SUCCESS;

    if (p_myprofile->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        ble_gatts_hvx_params_t hvx_params;
        uint8_t encoded_value[MAX_MYCHARACTERISTIC_LEN];
        uint16_t hvx_len;

        // Initialize value struct.
        memset(&hvx_params, 0, sizeof(hvx_params));

        hvx_len           = mycharacteristic_encode(p_mycharacteristic, encoded_value);
        hvx_params.handle = p_myprofile->mycharacteristic_handles.value_handle;
        hvx_params.type   = BLE_GATT_HVX_INDICATION;
        hvx_params.p_len  = &hvx_len;
        hvx_params.offset = 0;
        hvx_params.p_data = encoded_value;

        err_code = sd_ble_gatts_hvx(p_myprofile->conn_handle, &hvx_params);
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
}

uint32_t ble_my_service_value_update(ble_myprofile_t * p_myprofile, uint8_t data)
{
	uint32_t err_code = NRF_SUCCESS;
	
	ble_myprofile_evt_t ble_myprofile_evt;
	ble_gatts_value_t gatts_value;	
	ble_myprofile_t ble_myprofile;
	ble_myprofile_mycharacteristic_t * p_mycharacteristic;

	
	memset(&gatts_value, 0, sizeof(gatts_value));

	gatts_value.len     = sizeof(uint8_t);
	gatts_value.offset  = 0;
	gatts_value.p_value = &data;

	err_code = sd_ble_gatts_value_set(p_myprofile->conn_handle, 
																		p_myprofile->mycharacteristic_handles.value_handle, 
																		&gatts_value);
	
	if (p_myprofile->conn_handle != BLE_CONN_HANDLE_INVALID)
	{
		ble_gatts_hvx_params_t hvx_params;
		// Initialize value struct.
		memset(&hvx_params, 0, sizeof(hvx_params));
		
		hvx_params.handle = p_myprofile->mycharacteristic_handles.value_handle;
		hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = gatts_value.offset;
		hvx_params.p_len  = &gatts_value.len;
		hvx_params.p_data = gatts_value.p_value;

		err_code = sd_ble_gatts_hvx(p_myprofile->conn_handle, &hvx_params);
	}
	return err_code;
}
